<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;
use Util\Repositories;
use Util\NovoCodigo;

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil, Utils };
use Ems\Repository\{ PeriodoRepository, SituacaoFuncionalRepository };

class SituacaoFuncionalService {
    /** @var Environment */
    protected $environment;

    /** @var EntityManager */
    protected $entityManager;

    /** @var SituacaoFuncionalRepository */
    protected $situacaoFuncionalRepository;

    /** @var PeriodoRepository */
    protected $periodoRepository;
    
    public const NRSITUFUNCM_ATIVIDADE_NORMAL = 1;

    public function __construct(
        EntityManager                   $entityManager,
        Environment                     $environment,
        SituacaoFuncionalRepository     $situacaoFuncionalRepository,
        PeriodoRepository               $periodoRepository
    ) {
        $this->entityManager                = $entityManager;
        $this->environment                  = $environment;
        $this->situacaoFuncionalRepository  = $situacaoFuncionalRepository;
        $this->periodoRepository            = $periodoRepository;
    }

  /**
    * Rotina responsável por obter a última situação funcional de um determinado vínculo em um período de apuração ou dentro da competência. Caso
    * seja informado um período somente, a data de competência é obtida a partir dele
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaUltimaSituacaoFuncional
    */
    public function retornaUltimaSituacaoFuncional(
        int     $nrVinculom,
        string  $nrPeriodoApuracao = null,
        string  $mesCompetencia = null
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        if (empty($nrPeriodoApuracao) && empty($mesCompetencia)) {
            return false;
        }

        $dtCompetenciaSituacaoFuncional = $mesCompetencia;
        if (empty($mesCompetencia)) {
            $dtCompetenciaSituacaoFuncional = $this->periodoRepository->getDtmescompetencByPeriodo($nrOrg, $nrPeriodoApuracao);
        }

        return $this->situacaoFuncionalRepository->retornaUltimaSituacaoFuncional(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom,
            $dtCompetenciaSituacaoFuncional    
        );
    }
    
  /**
    * Rotina responsável por obter os períodos de estabilidade de um determinado vínculo
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaPeriodosEstabilidade
    */
    public function retornaPeriodosEstabilidade(
        int     $nrVinculom
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        return $this->situacaoFuncionalRepository->retornaPeriodosEstabilidade(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom
        );
    }

    public function setAlteEscala(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRALTEESCALA'] =
            $row['NRALTEESCALA'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTEESCALA', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRVINCULOM']           = (int) $row['NRVINCULOM'];
        $params['DTINIESCALA']          = (string) $row['DTINIESCALA'];
        $params['DTFIMESCALA']          = $row['DTFIMESCALA'] ?? null;
        $params['NRESCALATRABM']        = (int) $row['NRESCALATRABM'];
        $params['NRHORDIAM']            = $row['NRHORDIAM'] ?? null;
        $params['NRMOTIVOESCALA']       = $row['NRMOTIVOESCALA'] ?? null;
        $params['DSOBSERVACAO']         = $row['DSOBSERVACAO'] ?? null;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;
        $params['NRTURNO']              = $row['NRTURNO'] ?? null;
        $params['QTHRLIMITESEPHEXTRA']  = $row['QTHRLIMITESEPHEXTRA'] ?? null;

        $this->situacaoFuncionalRepository->setAlteEscala($params);

        return $params['NRALTEESCALA'];
    }

    public function setAlteSituacaoFuncional(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRALTESITUFUNC'] =
            $row['NRALTESITUFUNC'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTESITUFUNC', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRMOTIVOAFASTA']       = $row['NRMOTIVOAFASTA'] ?? null;
        $params['NRPROXSITUFUNC']       = $row['NRPROXSITUFUNC'] ?? null;
        $params['CDTABECDI']            = $row['CDTABECDI'] ?? null;
        $params['CDDIAGNOST']           = $row['CDDIAGNOST'] ?? null;
        $params['NRSITUFUNCM']          = (int) $row['NRSITUFUNCM'];
        $params['NRVINCULOM']           = (int) $row['NRVINCULOM'];
        $params['DTINISITUFUNC']        = (string) $row['DTINISITUFUNC'];
        $params['NRMOTIVOAFASTA']       = $row['NRMOTIVOAFASTA'] ?? null;
        $params['DTFIMSITUFUNC']        = $row['DTFIMSITUFUNC'] ?? null;
        $params['DTVERIFSITUFUNC']      = $row['DTVERIFSITUFUNC'] ?? null;
        $params['DTPREVIRETOSITU']      = $row['DTPREVIRETOSITU'] ?? null;
        $params['DTPRORRSITUFUNC']      = $row['DTPRORRSITUFUNC'] ?? null;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;
        $params['DSOBSALTSITFUNC']      = $row['DSOBSALTSITFUNC'] ?? null;
        $params['NRQTDIASITUFUNC']      = $row['NRQTDIASITUFUNC'] ?? null;
        $params['DTESTABSITUFUNC']      = $row['DTESTABSITUFUNC'] ?? null;
        $params['NRPESSOA']             = $row['NRPESSOA'] ?? null;
        $params['QTDIASATESTADOEMPR']   = $row['QTDIASATESTADOEMPR'] ?? null;

        $this->situacaoFuncionalRepository->setAlteSituacaoFuncional($params);

        return $params['NRALTESITUFUNC'];
    }
    
    /**
     *  Rotina responsável pelo cadastro de alteracao de situacao no momento do cadastro de ocorrencias
     * 
     * 
     *  @param $nrVinculom
     *  @param $alteredRow      array()
     *  @param $previousRow     array()
     *  @param $operation       IN [Utils::DATA_OPERATION_INSERT, Utils::DATA_OPERATION_UPDATE, Utils::DATA_OPERATION_DELETE]
     */
    public function afterAlterOcorrenciaSetAlteSituFunc(
        $nrVinculom,
        $alteredRow     = array(),
        $previousRow    = array(),
        $operation      = Utils::DATA_OPERATION_INSERT
    ) {
        
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $dateInclusao = new \DateTime();
        
        try {
        
            $nrMotivoOcorFreq = $operation == Utils::DATA_OPERATION_DELETE ? $previousRow['NRMOTIVOOCORFREQ'] ?? null : $alteredRow['NRMOTIVOOCORFREQ'] ?? null;
            if (empty($nrMotivoOcorFreq)) {
                return false;
            }
            
            // Busca pelas situacoes funcionais relacionadas ao motivo de ocorrencia cadastrado
            $associacoesMotivoSituFunc = $this->situacaoFuncionalRepository->getMotivoSituFunc($nrOrg, $nrMotivoOcorFreq);
            
            if (count($associacoesMotivoSituFunc) < 1) {
                return;
            }
            
            // Validations
            if (!empty($previousRow) && !empty($previousRow['DTINICIOPROGOCOR']) && strlen($previousRow['DTINICIOPROGOCOR']) > 10) {
                $previousRow['DTINICIOPROGOCOR'] = substr($previousRow['DTINICIOPROGOCOR'], 0, 10);
            }
            
            if (!empty($previousRow) && !empty($previousRow['DTFIMPROGOCOR']) && strlen($previousRow['DTFIMPROGOCOR']) > 10) {
                $previousRow['DTFIMPROGOCOR'] = substr($previousRow['DTFIMPROGOCOR'], 0, 10);
            }
            
            $this->entityManager->beginTransaction();
            
            switch ($operation) {
                case Utils::DATA_OPERATION_INSERT:
                    
                    $dataIniSituFuncAltered = DateUtil::getDataDeString($alteredRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dataFimSituFuncAltered = !empty($alteredRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($alteredRow['DTFIMPROGOCOR'], DateUtil::FORMATO_BRASILEIRO, true) : null;
                    
                    $quantidadeDias = null;
                    if (!empty($dataFimSituFuncAltered)) {
                        $quantidadeDias = DateUtil::getQtdeDiasEntre($dataIniSituFuncAltered, $dataFimSituFuncAltered) + 1;
                    }
    
                    // Insere uma alteracao de situacao funcional para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Insere situacao funcional relacionada
                        $row = array(
                            'NRORG'             => $nrOrg,
                            'NRVINCULOM'        => $nrVinculom,
                            'NRSITUFUNCM'       => $associacaoMotivoSituFunc['NRSITUFUNCM'],
                            'NRMOTIVOAFASTA'    => $associacaoMotivoSituFunc['NRMOTIVOAFASTA'],
                            'DTINISITUFUNC'     => $dataIniSituFuncAltered->format(DateUtil::FORMATO_DATA),
                            'DTFIMSITUFUNC'     => empty($dataFimSituFuncAltered) ? null : $dataFimSituFuncAltered->format(DateUtil::FORMATO_DATA),
                            'NRQTDIASITUFUNC'   => $quantidadeDias,
                            'CDTABECDI'         => $alteredRow['CDTABECDI']     ?? null,
                            'CDDIAGNOST'        => $alteredRow['CDDIAGNOST']    ?? null,
                            'NRPESSOA'          => $alteredRow['NRPESSOA']      ?? null,
                        );
                        
                        $this->setAlteSituacaoFuncional($row);
                        
                        // Insere situacao funcional de trabalho
                        if (!empty($dataFimSituFuncAltered)) {
                            $dataFimSituFuncAlteredProxTrab = DateUtil::adicionaIntervalo($dataFimSituFuncAltered, 1, DateUtil::DIA);
                        
                            $row = array(
                                'NRORG'         => $nrOrg,
                                'NRVINCULOM'    => $nrVinculom,
                                'NRSITUFUNCM'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL,
                                'DTINISITUFUNC' => $dataFimSituFuncAlteredProxTrab->format(DateUtil::FORMATO_DATA),
                                'DTFIMSITUFUNC' => null,
                                'CDTABECDI'     => $alteredRow['CDTABECDI']     ?? null,
                                'CDDIAGNOST'    => $alteredRow['CDDIAGNOST']    ?? null,
                                'NRPESSOA'      => $alteredRow['NRPESSOA']      ?? null,
                            );
                            
                            $this->setAlteSituacaoFuncional($row);   
                        }
                    }
                    
                    break;
                case Utils::DATA_OPERATION_UPDATE:
                    
                    // Por enquanto n tratar quando altera o motivo, a trigger n faz isso. Ver se teria como fazer, pq pode ser melhor;
                    
                    // Edicao ou remocao. Tratar FORMATO_DATAHORA
                    $dataIniSituFuncPrevious = DateUtil::getDataDeString($previousRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_DATA, true);
                    $dataFimSituFuncPrevious = !empty($previousRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($previousRow['DTFIMPROGOCOR'], DateUtil::FORMATO_DATA, true) : null;
                    
                    $dataIniSituFuncAltered = DateUtil::getDataDeString($alteredRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_DATA, true);
                    $dataFimSituFuncAltered = !empty($alteredRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($alteredRow['DTFIMPROGOCOR'], DateUtil::FORMATO_DATA, true) : null;
                    
                    $quantidadeDias = null;
                    if (!empty($dataFimSituFuncAltered)) {
                        $quantidadeDias = DateUtil::getQtdeDiasEntre($dataIniSituFuncAltered, $dataFimSituFuncAltered) + 1;
                    }
                    
                    // Atualiza alteracoes de situacao funcional existentes para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Altera situacao funcional relacionada
                        $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                            array(
                                'nrvinculom'    => $nrVinculom,
                                'dtinisitufunc' => $dataIniSituFuncPrevious,
                                'dtfimsitufunc' => $dataFimSituFuncPrevious,
                                'nrsitufuncm'   => $associacaoMotivoSituFunc['NRSITUFUNCM']
                            )
                        );
                        
                        foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                            $alteracaoSituacaoFuncional->setDtinisitufunc($dataIniSituFuncAltered);
                            $alteracaoSituacaoFuncional->setDtfimsitufunc($dataFimSituFuncAltered);
                            $alteracaoSituacaoFuncional->setNrmotivoafasta($associacaoMotivoSituFunc['NRMOTIVOAFASTA']);
                            $alteracaoSituacaoFuncional->setNrqtdiasitufunc($quantidadeDias);
                            $alteracaoSituacaoFuncional->setCdtabecdi($alteredRow['CDTABECDI']      ?? null);
                            $alteracaoSituacaoFuncional->setCddiagnost($alteredRow['CDDIAGNOST']    ?? null);
                            $alteracaoSituacaoFuncional->setNrpessoa($alteredRow['NRPESSOA']        ?? null);
                            $alteracaoSituacaoFuncional->setNrorgultatu($nrOrg);
                            $alteracaoSituacaoFuncional->setCdoperultatu($cdOperador);
                            $alteracaoSituacaoFuncional->setDtultatu($dateInclusao);
                            
                            $this->entityManager->persist($alteracaoSituacaoFuncional);
                        }
                        
                        // Salva alteracoes na(s) situacao(oes) criada(s)
                        $this->entityManager->flush();
                        
                        // Altera situacao funcional de trabalho futuro
                        $dataFimSituFuncPreviousProxTrab = !empty($dataFimSituFuncPrevious) ? DateUtil::adicionaIntervalo($dataFimSituFuncPrevious, 1, DateUtil::DIA) : null;
                        $dataFimSituFuncAlteredProxTrab = !empty($dataFimSituFuncAltered) ? DateUtil::adicionaIntervalo($dataFimSituFuncAltered, 1, DateUtil::DIA) : null;
                        
                        // Verifica se é necessario criar/alterar proximas situacoes de trabalho
                        if (empty($dataFimSituFuncPreviousProxTrab) && empty($dataFimSituFuncAlteredProxTrab)) {
                            continue;
                        }
                        
                        // Verifica se devera criar a proxima situacao funcional, pois uma nova data fim foi associada a ocorrencia
                        if (empty($dataFimSituFuncPreviousProxTrab)) {
                            $row = array(
                                'NRORG'         => $nrOrg,
                                'NRVINCULOM'    => $nrVinculom,
                                'NRSITUFUNCM'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL,
                                'DTINISITUFUNC' => $dataFimSituFuncAlteredProxTrab->format(DateUtil::FORMATO_DATA),
                                'DTFIMSITUFUNC' => null,
                                'CDTABECDI'     => $alteredRow['CDTABECDI']     ?? null,
                                'CDDIAGNOST'    => $alteredRow['CDDIAGNOST']    ?? null,
                                'NRPESSOA'      => $alteredRow['NRPESSOA']      ?? null,
                            );
                            
                            $this->setAlteSituacaoFuncional($row);
                        } else {
                            // Verifica se devera editar/remover a proxima situacao funcional
                            $alteracoesProxSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                                array(
                                    'nrvinculom'    => $nrVinculom,
                                    'dtinisitufunc' => $dataFimSituFuncPreviousProxTrab,
                                    'dtfimsitufunc' => null,
                                    'nrsitufuncm'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL
                                )
                            );
                            
                            foreach($alteracoesProxSituacaoFuncional as $alteracaoSituacaoFuncional) {
                                
                                // Caso a data fim for retirada, a situacao funcional de trabalho apos a situacao relacionada eh excluida
                                if (empty($dataFimSituFuncAlteredProxTrab)) {
                                    $this->entityManager->remove($alteracaoSituacaoFuncional);    
                                } else {
                                    // Caso contrario, a data de inicio da proxima situacao de trabalho eh somente ajustada
                                    $alteracaoSituacaoFuncional->setDtinisitufunc($dataFimSituFuncAlteredProxTrab);
                                    $alteracaoSituacaoFuncional->setNrorgultatu($nrOrg);
                                    $alteracaoSituacaoFuncional->setCdoperultatu($cdOperador);
                                    $alteracaoSituacaoFuncional->setDtultatu($dateInclusao);
                                    $this->entityManager->persist($alteracaoSituacaoFuncional);
                                }
                            }
                        }
                        
                        // Salva alteracoes realizadas na proxima situacao de trabalho
                        $this->entityManager->flush();
                    }
                    
                    break;
                case Utils::DATA_OPERATION_DELETE:
                    
                    $dataIniSituFuncPrevious = DateUtil::getDataDeString($previousRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_DATA, true);
                    $dataFimSituFuncPrevious = !empty($previousRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($previousRow['DTFIMPROGOCOR'], DateUtil::FORMATO_DATA, true) : null;
                    
                    // Atualiza alteracoes de situacao funcional existentes para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Remove a situacao relacionada
                        $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                            array(
                                'nrvinculom'    => $nrVinculom,
                                'dtinisitufunc' => $dataIniSituFuncPrevious,
                                'dtfimsitufunc' => $dataFimSituFuncPrevious,
                                'nrsitufuncm'   => $associacaoMotivoSituFunc['NRSITUFUNCM']
                            )
                        );
                        
                        foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                            $this->entityManager->remove($alteracaoSituacaoFuncional);
                        }
                        
                        // Remove a proxima situacao de trabalho, caso a ocorrencia possua data fim
                        if (!empty($dataFimSituFuncPrevious)) {
                            $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                                array(
                                    'nrvinculom'    => $nrVinculom,
                                    'dtinisitufunc' => DateUtil::adicionaIntervalo($dataFimSituFuncPrevious, 1, DateUtil::DIA),
                                    'dtfimsitufunc' => null,
                                    'nrsitufuncm'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL
                                )
                            );
                            
                            foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                                $this->entityManager->remove($alteracaoSituacaoFuncional);
                            }
                        }
                        
                        $this->entityManager->flush();
                    }
                    break;
                default:
                    return false;
            }
            
            $this->entityManager->commit();
            return true;
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw new \Exception('Atenção! Houve um problema manipular as alterações de situação funcional. ' . $e->getMessage());
        }
    }
}
